# sb::File
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::File;
use strict;
use sb ();
use sb::File::MimeType ();
use sb::Object ();
use base qw( sb::Object );
use vars qw( $VERSION );
$VERSION = '0.08';
# 0.08 [2007/07/20] use sb->string istead of sb::Language
# 0.07 [2007/07/15] tidied up code
# 0.06 [2007/07/09] fixed _error to use "sb" instead of "Spica"
# 0.05 [2006/07/04] got rid of "return( undef )" and "return undef"
# 0.04 [2006/03/31] added get_text and _error
# 0.03 [2006/03/17] changed bracket rule
# 0.02 [2006/03/12] added set_data to write data to file
# 0.01 [2005/10/28] moved _content_type and _extension to sb::File::MimeType
# 0.00 [2005/10/27] generated
# --------------------------------------
# // defines / static variables
sub MIMEENCODE_BUFFERSIZE (){ 1026 };
sub DEFAULT_PERMISSION    (){ 0666 };
sub BASE_UNIT             (){ 1024 };
my $aTypeDictionary = 'sb::File::MimeType';
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'path'     => undef,
		'name'     => undef,
		'fullpath' => undef,
		'param'    => {},
		@_
	);
	if ($param{'fullpath'} ne '')
	{
		if ($param{'fullpath'} =~ m!/$!)
		{
			$param{'path'} = $param{'fullpath'};
		}
		else
		{
			my @path = split('/',$param{'fullpath'});
			$param{'name'} = pop(@path);
			$param{'path'} = (@path) ? join('/',@path) . '/' : './';
		}
	}
	$param{'path'} = &_check_path($param{'path'});
	delete $param{'fullpath'};
	return $self->SUPER::initialize(%param);
}
# --------------------------------------
# // public functions - class method
sub dictionary_class
{
	my $class = shift;
	my $new_class = shift if (@_);
	if ($new_class and $new_class ne $aTypeDictionary)
	{
		$aTypeDictionary = $new_class if ( &sb::load_module($new_class) );
	}
	return $aTypeDictionary;
}
# --------------------------------------
# // public functions - file name
sub path
{
	my $self = shift;
	$self->{'path'} = &_check_path(shift) if (@_);
	return $self->{'path'};
}
sub name
{
	my $self = shift;
	$self->{'name'} = shift if (@_);
	return $self->{'name'};
}
sub name_without_ext
{
	my $self = shift;
	my $name = $self->name;
	$name =~ s/(.*)\.(.*)$/$1/;
	return $name;
}
sub fullpath
{
	my $self = shift;
	return $self->path . $self->name;
}
sub extension
{
	my $self = shift;
	return '.' . (split('\.',$self->name))[-1];
}
sub supported_extension
{
	my $self = shift;
	return $self->dictionary_class->extension($self->content_type);
}
# --------------------------------------
# // public functions - file attribute
sub filesize
{
	my $self = shift;
	return -s $self->fullpath;
}
sub size_with_unit
{
	my $self = shift;
	my $size = $self->filesize;
	return $size . 'byte'  if ($size < 2);
	return $size . 'bytes' if ($size < BASE_UNIT);
	return sprintf("%.1fKB", $size / BASE_UNIT) if ($size < 10 * BASE_UNIT);
	return int($size / BASE_UNIT) . "KB" if ($size < BASE_UNIT * BASE_UNIT);
	return sprintf("%.1fMB", $size / (BASE_UNIT * BASE_UNIT));
}
sub is_readable
{
	my $self = shift;
	return -r $self->fullpath;
}
sub is_writable
{
	my $self = shift;
	return -w $self->fullpath;
}
sub is_dir
{
	my $self = shift;
	return if ($self->name ne '');
	return -d $self->fullpath;
}
sub exists
{
	my $self = shift;
	return -e $self->fullpath;
}
sub content_type
{
	my $self = shift;
	return $self->dictionary_class->content_type($self->name);
}
# --------------------------------------
# // public functions - file access
sub rename
{
	my $self = shift;
	my %param = (
		'path' => $self->path,
		'name' => $self->name,
		@_
	);
	$param{'path'} = &_check_path($param{'path'});
	my $new_name = $param{'path'} . $param{'name'};
	return if ($self->fullpath eq $new_name);
	if ( ( $self->exists and rename($self->fullpath,$new_name)) # renamed successfully
	  or (!$self->exists and -e $new_name) ) # already renamed
	{ # update instance variables as well.
		$self->path($param{'path'});
		$self->name($param{'name'});
		return 1;
	}
	return;
}
sub remove
{
	my $self = shift;
	return if (!$self->exists or !$self->is_writable);
	unlink($self->fullpath);
}
sub touch
{
	my $self = shift;
	my $permission = shift;
	$permission = DEFAULT_PERMISSION if (!$permission);
	my $file = $self->fullpath;
	eval {
		die("The file already exists.\n") if ($self->exists);
		open(EMPTY,">$file") or die("Failed to create the file.\n");
		binmode(EMPTY);
		print EMPTY undef;
		close(EMPTY);
		$self->set_permission($permission);
	};
	return $self->_error if ($@);
	return 1;
}
sub get_as_mime
{
	my $self = shift;
	my %param = (
		'buffersize' => MIMEENCODE_BUFFERSIZE,
		@_
	);
	my $data = undef;
	eval {
		my $file = $self->fullpath;
		require 'mimeutil.pl';
		die("The file does not exist.\n") if (!$self->exists);
		die("It is a directory.\n") if ($self->is_dir);
		die("The file is not readable.\n") if (!$self->is_readable);
		open(FILEIN,"<$file") or die("Failed to open the file.\n");
		binmode(FILEIN);
		while ( read(FILEIN, $_, $param{'buffersize'}) )
		{
			$data .= &mimeutil::bodyencode($_,'b64');
		}
		$data .= &mimeutil::benflush('b64');
		close(FILEIN);
	};
	return $self->_error if ($@);
	return $data;
}
sub set_data
{
	my $self = shift;
	my $data = shift;
	eval
	{
		my $file = $self->fullpath;
		die("The data is undefined.\n") if (! defined($data));
		die("The file does not exist.\n") if (!$self->exists);
		die("It is a directory.\n") if ($self->is_dir);
		die("The file is not writable.\n") if (!$self->is_writable);
		open(FILEOUT,">$file") or die("Failed to open the file.\n");
		binmode(FILEOUT);
		print FILEOUT $data;
		close(FILEOUT);
	};
	return $self->_error if ($@);
	return 1;
}
sub get_text
{
	my $self = shift;
	my $data = undef;
	eval {
		my $file = $self->fullpath;
		die("The file does not exist.\n") if (!$self->exists);
		die("It is a directory.\n") if ($self->is_dir);
		die("The file is not readable.\n") if (!$self->is_readable);
		open(FILEIN,"<$file") or die("Failed to open the file.\n");
		binmode(FILEIN);
		while (<FILEIN>)
		{
			$_ =~ tr/\x0D\x0A//d;
			$data .= $_ . "\n";
		}
		close(FILEIN);
	};
	return $self->_error if ($@);
	return $data;
}
sub set_permission
{
	my $self = shift;
	my $permission = shift;
	$permission = DEFAULT_PERMISSION if (!$permission);
	chmod($permission,$self->fullpath);
	return;
}
# --------------------------------------
# // private functions
sub _check_path
{
	my $path = shift;
	$path .= '/' if ($path !~ m!/$!);
	return $path;
}
sub _error
{
	my $self = shift;
	$@ =~ s/\n$//;
	$@ = sb->string($@) . ' : ' . $self->fullpath;
	return;
}
1;
__END__
