# sb::Local
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Local;
use strict;
use sb ();
use sb::Time ();
use sb::Object ();
use base qw( sb::RequestObject );
use vars qw( $VERSION %DATABASE );
$VERSION = '0.07';
# 0.07 [2007/07/20] changed to call proper sb::Time method (data) and to use sb->string
# 0.06 [2006/03/22] added _timezone, _dst, changed timezone
# 0.05 [2006/03/17] changed bracket rule
# 0.04 [2006/03/13] changed timezone_base to fix a bug
# 0.03 [2005/12/28] changed _easter to fix a bug
# 0.02 [2005/11/02] changed timezone_base to calculate correctly, changed _weekday to handle the last weekday
# 0.01 [2005/10/27] added _easter
# 0.00 [2005/10/25] generated
# --------------------------------------
# // defines / static variables
# --------------------------------------
# // public functions - common
sub timezone
{
	my $self = shift;
	my $obj  = shift;
	my $name = shift;
	return $self->timezone_base(
		'current'  => $obj,
		'name'     => $name,
	);
}
sub holidays
{
	my $self = shift;
	my $year = shift;
	my $easter = join('',$self->easter($year));
	return $self->holidays_base(
		'0101'  => 'New Year\'s Day',
		$easter => 'Easter Day',
		'1225'  => 'Christmas Day',
		'1226'  => 'Boxing Day',
	);
}
# --------------------------------------
# // public functions
sub timezone_base
{
	my $self = shift;
	my %param = (
		'current'  => undef,   # check time (sb::Time object or Unixtime)
		'name'     => undef,   # name
		'standard' => '+0000', # standard timezone
		'adjust'   => '+0000', # day-light saving adjustment
		'start'    => undef,   # nth; weekday; on month; at time (+0000 - +2359)
		'end'      => undef,   # nth; weekday; on month; at time (+0000 - +2359)
		@_
	);
	if ($param{'name'})
	{
		my $data = $self->_timezone_data($param{'name'});
		if ($data)
		{
			$param{'standard'} = $data->[0];
			$param{'adjust'} = $data->[1];
			$param{'start'} = $data->[2];
			$param{'end'} = $data->[3];
		}
	}
	return $param{'standard'} if ($param{'adjust'} eq '+0000');
	my $saving = $self->_dst($param{'standard'},$param{'adjust'});
	my $now  = $self->_get_time($param{'current'});
	my $year = $self->_get_year($now);
	my @period = ();
	foreach my $key ('start','end')
	{
		my @val = split(';',$param{$key});
		my $unixTime = sb::Time->new(
			'year' => $year,
			'mon'  => $val[2],
			'day'  => $self->_weekday('year'=>$year,'mon'=>$val[2],'nth'=>$val[0],'week'=>$val[1]),
			'zone' => ($key eq 'start') ? $param{'standard'} : $saving,
		);
		$val[3] = '+' . $1 if ($val[3] =~ /^(\d\d\d\d)$/);
		push(@period,$unixTime->data() + sb::Time->diff_timezone($val[3],'0'));
	}
	return $param{'standard'} if ($period[0] == $period[1]);
	my $flag = ($period[0] >= $period[1]);
	my $checker = ($now < $period[0] and $period[1] <= $now) ? $flag : !$flag;
	return $checker ? $param{'standard'} : $saving;
}
sub holidays_base
{
	my $self = shift;
	my %list = @_;
	foreach my $date (keys(%list))
	{
		$list{$date} = sb->string($list{$date});
	}
	return \%list;
}
# --------------------------------------
# // private functions
sub _pad0
{
	return &sb::Time::_pad0(@_[1]);
}
sub _weekday
{
	my $self = shift;
	my %param = (
		'year' => 1970,
		'mon'  => 1,
		'nth'  => 1, # between 1 and 5, or -1 => -1 means the last weekday
		'week' => 0, # 0 => Sun, ..., 6 => Sat
		@_
	);
	my $nth = ($param{'nth'} == -1 or $param{'nth'} > 5) ? 5
	        : ($param{'nth'} < 1) ? 1
	        : $param{'nth'};
	my $first = sb::Time->get_weekday('year'=>$param{'year'},'mon'=>$param{'mon'},'day'=>1);
	my $day = 1 + ($param{'week'} - $first) % 7 + 7 * ($nth - 1);
	$day -= 7 if ($day > sb::Time->get_lastday('year'=>$param{'year'},'mon'=>$param{'mon'}));
	return $day;
}
sub _get_year
{
	my $self = shift;
	return (gmtime($_[0]))[5] + 1900;
}
sub _get_time
{
	my $self = shift;
	my $obj  = shift;
	return (ref($obj) eq 'sb::Time') ? $obj->data() 
	     : ($obj =~ /^\d+$/) ? $obj
	     : time();
}
sub _easter
{ # from Date::Easter 1.14 / Copyright (C) 2002 Rich Bowen <rbowen@rcbowen.com>
	my $self = shift;
	my $year = shift;
	my ($mon,$day);
	my ($G, $C, $H, $I, $J, $L);
	$G = $year % 19;
	$C = int($year / 100);
	$H = ( $C - int( $C / 4 ) - int( ( 8 * $C ) / 25 ) + 19 * $G + 15 ) % 30;
	$I = $H - int( $H / 28 ) * (1 - int( $H / 28 ) * int( 29 / ( $H + 1 ) ) * int( ( 21 - $G ) / 11 ));
	$J = ( $year + int( $year / 4 ) + $I + 2 - $C + int( $C / 4 ) ) % 7;
	$L = $I - $J;
	$mon = $self->_pad0(3 + int( ( $L + 40 ) / 44 ));
	$day = $self->_pad0($L + 28 - ( 31 * int( $mon / 4 ) ));
	return ($mon,$day);
}
sub _dst
{
	my $self = shift;
	my ($zone,$adj) = @_;
	my $dst = undef;
	my @std = &_zone($zone);
	my @dif = &_zone($adj);
	$std[0] += $dif[0];
	$std[1] += $dif[1];
	$std[0]++ if ($std[1] >= 60);
	$std[1] %= 60;
	$dst = sprintf('%02d%02d',@std);
	$dst = '-0' . $1 . $2 if ($dst < 0 and $dst =~ /^-(\d)(\d\d)$/);
	$dst = '+' . $1 if ($dst =~ /^(\d\d\d\d)$/);
	return $dst;
	sub _zone
	{
		my $zone = shift;
		my ($h,$m) = ($zone =~ /(\d\d)(\d\d)$/);
		$h *= -1 if ($zone =~ /^-/);
		return ($h,$m);
	}
}
sub _timezone_data
{
	my $self = shift;
	my $name = shift;
	return if (!$name);
	return $DATABASE{$name};
}
%DATABASE = (# name              => [offset,dst adj.,dst bgn,dst end,std,dst],
	'Africa/Abidjan'               => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Accra'                 => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Addis_Ababa'           => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Algiers'               => ['+0100','+0000',undef,undef,'CET',undef],
	'Africa/Asmera'                => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Bamako'                => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Bangui'                => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Banjul'                => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Bissau'                => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Blantyre'              => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Brazzaville'           => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Bujumbura'             => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Cairo'                 => ['+0200','+0100','-1;5;4;+0000','-1;5;9;+0000','EET','EEST'],
	'Africa/Casablanca'            => ['+0000','+0000',undef,undef,'WET',undef],
	'Africa/Ceuta'                 => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Africa/Conakry'               => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Dakar'                 => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Dar_es_Salaam'         => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Djibouti'              => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Douala'                => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/El_Aaiun'              => ['+0000','+0000',undef,undef,'WET',undef],
	'Africa/Freetown'              => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Gaborone'              => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Harare'                => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Johannesburg'          => ['+0200','+0000',undef,undef,'SAST',undef],
	'Africa/Kampala'               => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Khartoum'              => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Kigali'                => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Kinshasa'              => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Lagos'                 => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Libreville'            => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Lome'                  => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Luanda'                => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Lubumbashi'            => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Lusaka'                => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Malabo'                => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Maputo'                => ['+0200','+0000',undef,undef,'CAT',undef],
	'Africa/Maseru'                => ['+0200','+0000',undef,undef,'SAST',undef],
	'Africa/Mbabane'               => ['+0200','+0000',undef,undef,'SAST',undef],
	'Africa/Mogadishu'             => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Monrovia'              => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Nairobi'               => ['+0300','+0000',undef,undef,'EAT',undef],
	'Africa/Ndjamena'              => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Niamey'                => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Nouakchott'            => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Ouagadougou'           => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Porto-Novo'            => ['+0100','+0000',undef,undef,'WAT',undef],
	'Africa/Sao_Tome'              => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Timbuktu'              => ['+0000','+0000',undef,undef,'GMT',undef],
	'Africa/Tripoli'               => ['+0200','+0000',undef,undef,'EET',undef],
	'Africa/Tunis'                 => ['+0100','+0000',undef,undef,'CET',undef],
	'Africa/Windhoek'              => ['+0100','+0100','1;0;9;+0200','1;0;4;+0200','WAT','WAST'],
	'America/Adak'                 => ['-1000','+0100','1;0;4;+0200','-1;0;10;+0200','HAST','HADT'],
	'America/Anchorage'            => ['-0900','+0100','1;0;4;+0200','-1;0;10;+0200','AKST','AKDT'],
	'America/Anguilla'             => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Antigua'              => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Araguaina'            => ['-0300','+0100','2;0;10;+0000','3;0;2;+0000','BRT','BRST'],
	'America/Aruba'                => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Asuncion'             => ['-0400','+0100','1;0;10;+0000','1;0;3;+0000','PYT','PYST'],
	'America/Barbados'             => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Belem'                => ['-0300','+0000',undef,undef,'BRT',undef],
	'America/Belize'               => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Boa_Vista'            => ['-0400','+0000',undef,undef,'AMT',undef],
	'America/Bogota'               => ['-0500','+0000',undef,undef,'COT',undef],
	'America/Boise'                => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/Buenos_Aires'         => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Cambridge_Bay'        => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/Cancun'               => ['-0600','+0100','1;0;5;+0200','-1;0;9;+0200','CST','CDT'],
	'America/Caracas'              => ['-0400','+0000',undef,undef,'VET',undef],
	'America/Catamarca'            => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Cayenne'              => ['-0300','+0000',undef,undef,'GFT',undef],
	'America/Cayman'               => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Chicago'              => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/Chihuahua'            => ['-0700','+0100','1;0;5;+0200','-1;0;9;+0200','MST','MDT'],
	'America/Cordoba'              => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Costa_Rica'           => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Cuiaba'               => ['-0400','+0100','2;0;10;+0000','3;0;2;+0000','AMT','AMST'],
	'America/Curacao'              => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Danmarkshavn'         => ['+0000','+0000',undef,undef,'GMT',undef],
	'America/Dawson'               => ['-0800','+0100','1;0;4;+0200','-1;0;10;+0200','PST','PDT'],
	'America/Dawson_Creek'         => ['-0700','+0000',undef,undef,'MST',undef],
	'America/Denver'               => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/Detroit'              => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Dominica'             => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Edmonton'             => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/Eirunepe'             => ['-0500','+0000',undef,undef,'ACT',undef],
	'America/El_Salvador'          => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Fortaleza'            => ['-0300','+0100','2;0;10;+0000','3;0;2;+0000','BRT','BRST'],
	'America/Glace_Bay'            => ['-0400','+0100','1;0;4;+0200','-1;0;10;+0200','AST','ADT'],
	'America/Godthab'              => ['-0300','+0100','-1;6;3;+2200','-1;6;10;+2300','WGT','WGST'],
	'America/Goose_Bay'            => ['-0400','+0100','1;0;4;+0001','-1;0;10;+0001','AST','ADT'],
	'America/Grand_Turk'           => ['-0500','+0100','1;0;4;+0000','-1;0;10;+0000','EST','EDT'],
	'America/Grenada'              => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Guadeloupe'           => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Guatemala'            => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Guayaquil'            => ['-0500','+0000',undef,undef,'ECT',undef],
	'America/Guyana'               => ['-0400','+0000',undef,undef,'GYT',undef],
	'America/Halifax'              => ['-0400','+0100','1;0;4;+0200','-1;0;10;+0200','AST','ADT'],
	'America/Havana'               => ['-0500','+0100','1;0;4;+0000','-1;0;10;+0100','CST','CDT'],
	'America/Hermosillo'           => ['-0700','+0000',undef,undef,'MST',undef],
	'America/Indiana/Indianapolis' => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Indiana/Knox'         => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Indiana/Marengo'      => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Indiana/Vevay'        => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Indianapolis'         => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Inuvik'               => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/Iqaluit'              => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Jamaica'              => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Jujuy'                => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Juneau'               => ['-0900','+0100','1;0;4;+0200','-1;0;10;+0200','AKST','AKDT'],
	'America/Kentucky/Louisville'  => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Kentucky/Monticello'  => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/La_Paz'               => ['-0400','+0000',undef,undef,'BOT',undef],
	'America/Lima'                 => ['-0500','+0000',undef,undef,'PET',undef],
	'America/Los_Angeles'          => ['-0800','+0100','1;0;4;+0200','-1;0;10;+0200','PST','PDT'],
	'America/Louisville'           => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Maceio'               => ['-0300','+0100','2;0;10;+0000','3;0;2;+0000','BRT','BRST'],
	'America/Managua'              => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Manaus'               => ['-0400','+0000',undef,undef,'AMT',undef],
	'America/Martinique'           => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Mazatlan'             => ['-0700','+0100','1;0;5;+0200','-1;0;9;+0200','MST','MDT'],
	'America/Mendoza'              => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Menominee'            => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/Merida'               => ['-0600','+0100','1;0;5;+0200','-1;0;9;+0200','CST','CDT'],
	'America/Mexico_City'          => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Miquelon'             => ['-0300','+0100','1;0;4;+0200','-1;0;10;+0200','PMST','PMDT'],
	'America/Monterrey'            => ['-0600','+0100','1;0;5;+0200','-1;0;9;+0200','CST','CDT'],
	'America/Montevideo'           => ['-0300','+0000',undef,undef,'UYT',undef],
	'America/Montreal'             => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Montserrat'           => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Nassau'               => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/New_York'             => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Nipigon'              => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Nome'                 => ['-0900','+0100','1;0;4;+0200','-1;0;10;+0200','AKST','AKDT'],
	'America/Noronha'              => ['-0200','+0000',undef,undef,'FNT',undef],
	'America/North_Dakota/Center'  => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/Panama'               => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Pangnirtung'          => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Paramaribo'           => ['-0300','+0000',undef,undef,'SRT',undef],
	'America/Phoenix'              => ['-0700','+0000',undef,undef,'MST',undef],
	'America/Port-au-Prince'       => ['-0500','+0000',undef,undef,'EST',undef],
	'America/Port_of_Spain'        => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Porto_Velho'          => ['-0400','+0000',undef,undef,'AMT',undef],
	'America/Puerto_Rico'          => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Rainy_River'          => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/Rankin_Inlet'         => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/Recife'               => ['-0300','+0100','2;0;10;+0000','3;0;2;+0000','BRT','BRST'],
	'America/Regina'               => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Rio_Branco'           => ['-0500','+0000',undef,undef,'ACT',undef],
	'America/Rosario'              => ['-0300','+0000',undef,undef,'ART',undef],
	'America/Santiago'             => ['-0400','+0100','2;0;10;+0000','2;0;3;+0000','CLT','CLST'],
	'America/Santo_Domingo'        => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Sao_Paulo'            => ['-0300','+0100','2;0;10;+0000','3;0;2;+0000','BRT','BRST'],
	'America/Scoresbysund'         => ['-0100','+0100','-1;0;3;+0000','-1;0;10;+0100','EGT','EGST'],
	'America/Shiprock'             => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'America/St_Johns'             => ['-0330','+0100','1;0;4;+0001','-1;0;10;+0001','NST','NDT'],
	'America/St_Kitts'             => ['-0400','+0000',undef,undef,'AST',undef],
	'America/St_Lucia'             => ['-0400','+0000',undef,undef,'AST',undef],
	'America/St_Louis'             => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0200','CST','CDT'],
	'America/St_Thomas'            => ['-0400','+0000',undef,undef,'AST',undef],
	'America/St_Vincent'           => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Swift_Current'        => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Tegucigalpa'          => ['-0600','+0000',undef,undef,'CST',undef],
	'America/Thule'                => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Thunder_Bay'          => ['-0500','+0100','1;0;4;+0200','-1;0;10;+0200','EST','EDT'],
	'America/Tijuana'              => ['-0800','+0100','1;0;4;+0200','-1;0;10;+0200','PST','PDT'],
	'America/Tortola'              => ['-0400','+0000',undef,undef,'AST',undef],
	'America/Vancouver'            => ['-0800','+0100','1;0;4;+0200','-1;0;10;+0200','PST','PDT'],
	'America/Whitehorse'           => ['-0800','+0100','1;0;4;+0200','-1;0;10;+0200','PST','PDT'],
	'America/Winnipeg'             => ['-0600','+0100','1;0;4;+0200','-1;0;10;+0300','CST','CDT'],
	'America/Yakutat'              => ['-0900','+0100','1;0;4;+0200','-1;0;10;+0200','AKST','AKDT'],
	'America/Yellowknife'          => ['-0700','+0100','1;0;4;+0200','-1;0;10;+0200','MST','MDT'],
	'Antarctica/Casey'             => ['+0800','+0000',undef,undef,'WST',undef],
	'Antarctica/Davis'             => ['+0700','+0000',undef,undef,'DAVT',undef],
	'Antarctica/DumontDUrville'    => ['+1000','+0000',undef,undef,'DDUT',undef],
	'Antarctica/Mawson'            => ['+0600','+0000',undef,undef,'MAWT',undef],
	'Antarctica/McMurdo'           => ['+1200','+0100','1;0;10;+0200','3;0;3;+0300','NZST','NZDT'],
	'Antarctica/Palmer'            => ['-0400','+0100','2;0;10;+0000','2;0;3;+0000','CLT','CLST'],
	'Antarctica/South_Pole'        => ['+1200','+0100','1;0;10;+0200','3;0;3;+0300','NZST','NZDT'],
	'Antarctica/Syowa'             => ['+0300','+0000',undef,undef,'SYOT',undef],
	'Antarctica/Vostok'            => ['+0600','+0000',undef,undef,'VOST',undef],
	'Arctic/Longyearbyen'          => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Asia/Aden'                    => ['+0300','+0000',undef,undef,'AST',undef],
	'Asia/Almaty'                  => ['+0600','+0100','-1;0;3;+0000','-1;0;10;+0000','ALMT','ALMST'],
	'Asia/Amman'                   => ['+0200','+0100','-1;4;3;+0000','-1;4;9;+0100','EET','EEST'],
	'Asia/Anadyr'                  => ['+1200','+0100','-1;0;3;+0200','-1;0;10;+0300','ANAT','ANAST'],
	'Asia/Aqtau'                   => ['+0400','+0100','-1;0;3;+0000','-1;0;10;+0000','AQTT','AQTST'],
	'Asia/Aqtobe'                  => ['+0500','+0100','-1;0;3;+0000','-1;0;10;+0000','AQTT','AQTST'],
	'Asia/Ashgabat'                => ['+0500','+0000',undef,undef,'TMT',undef],
	'Asia/Baghdad'                 => ['+0300','+0100','1;0;4;+0300','1;0;10;+0400','AST','ADT'],
	'Asia/Bahrain'                 => ['+0300','+0000',undef,undef,'AST',undef],
	'Asia/Baku'                    => ['+0400','+0100','-1;0;3;+0100','-1;0;10;+0100','AZT','AZST'],
	'Asia/Bangkok'                 => ['+0700','+0000',undef,undef,'ICT',undef],
	'Asia/Beirut'                  => ['+0200','+0100','-1;0;3;+0000','-1;0;10;+0000','EET','EEST'],
	'Asia/Bishkek'                 => ['+0500','+0100','-1;0;3;+0230','-1;0;10;+0230','KGT','KGST'],
	'Asia/Brunei'                  => ['+0800','+0000',undef,undef,'BNT',undef],
	'Asia/Calcutta'                => ['+0530','+0000',undef,undef,'IST',undef],
	'Asia/Choibalsan'              => ['+0900','+0000',undef,undef,'CHOT',undef],
	'Asia/Chongqing'               => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Colombo'                 => ['+0600','+0000',undef,undef,'LKT',undef],
	'Asia/Damascus'                => ['+0200','+0100','1;0;4;+0000','1;0;10;+0000','EET','EEST'],
	'Asia/Dhaka'                   => ['+0600','+0000',undef,undef,'BDT',undef],
	'Asia/Dili'                    => ['+0900','+0000',undef,undef,'TPT',undef],
	'Asia/Dubai'                   => ['+0400','+0000',undef,undef,'GST',undef],
	'Asia/Dushanbe'                => ['+0500','+0000',undef,undef,'TJT',undef],
	'Asia/Gaza'                    => ['+0200','+0100','3;5;4;+0000','3;5;10;+0000','EET','EEST'],
	'Asia/Harbin'                  => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Hong_Kong'               => ['+0800','+0000',undef,undef,'HKT',undef],
	'Asia/Hovd'                    => ['+0700','+0000',undef,undef,'HOVT',undef],
	'Asia/Irkutsk'                 => ['+0800','+0100','-1;0;3;+0200','-1;0;10;+0300','IRKT','IRKST'],
	'Asia/Istanbul'                => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Asia/Jakarta'                 => ['+0700','+0000',undef,undef,'WIT',undef],
	'Asia/Jayapura'                => ['+0900','+0000',undef,undef,'EIT',undef],
	'Asia/Jerusalem'               => ['+0200','+0100','1;0;4;+0100','1;0;10;+0100','IST','IDT'],
	'Asia/Kabul'                   => ['+0430','+0000',undef,undef,'AFT',undef],
	'Asia/Kamchatka'               => ['+1200','+0100','-1;0;3;+0200','-1;0;10;+0300','PETT','PETST'],
	'Asia/Karachi'                 => ['+0500','+0000',undef,undef,'PKT',undef],
	'Asia/Kashgar'                 => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Katmandu'                => ['+0545','+0000',undef,undef,'NPT',undef],
	'Asia/Krasnoyarsk'             => ['+0700','+0100','-1;0;3;+0200','-1;0;10;+0300','KRAT','KRAST'],
	'Asia/Kuala_Lumpur'            => ['+0800','+0000',undef,undef,'MYT',undef],
	'Asia/Kuching'                 => ['+0800','+0000',undef,undef,'MYT',undef],
	'Asia/Kuwait'                  => ['+0300','+0000',undef,undef,'AST',undef],
	'Asia/Macao'                   => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Magadan'                 => ['+1100','+0100','-1;0;3;+0200','-1;0;10;+0300','MAGT','MAGST'],
	'Asia/Manila'                  => ['+0800','+0000',undef,undef,'PHT',undef],
	'Asia/Muscat'                  => ['+0400','+0000',undef,undef,'GST',undef],
	'Asia/Nicosia'                 => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Asia/Novosibirsk'             => ['+0600','+0100','-1;0;3;+0200','-1;0;10;+0300','NOVT','NOVST'],
	'Asia/Omsk'                    => ['+0600','+0100','-1;0;3;+0200','-1;0;10;+0300','OMST','OMSST'],
	'Asia/Phnom_Penh'              => ['+0700','+0000',undef,undef,'ICT',undef],
	'Asia/Pontianak'               => ['+0700','+0000',undef,undef,'WIT',undef],
	'Asia/Pyongyang'               => ['+0900','+0000',undef,undef,'KST',undef],
	'Asia/Qatar'                   => ['+0300','+0000',undef,undef,'AST',undef],
	'Asia/Rangoon'                 => ['+0630','+0000',undef,undef,'MMT',undef],
	'Asia/Riyadh'                  => ['+0300','+0000',undef,undef,'AST',undef],
	'Asia/Saigon'                  => ['+0700','+0000',undef,undef,'ICT',undef],
	'Asia/Sakhalin'                => ['+1000','+0100','-1;0;3;+0200','-1;0;10;+0300','SAKT','SAKST'],
	'Asia/Samarkand'               => ['+0500','+0000',undef,undef,'UZT',undef],
	'Asia/Seoul'                   => ['+0900','+0000',undef,undef,'KST',undef],
	'Asia/Shanghai'                => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Singapore'               => ['+0800','+0000',undef,undef,'SGT',undef],
	'Asia/Taipei'                  => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Tashkent'                => ['+0500','+0000',undef,undef,'UZT',undef],
	'Asia/Tbilisi'                 => ['+0400','+0100','-1;0;3;+0000','-1;0;10;+0000','GET','GEST'],
	'Asia/Tehran'                  => ['+0330','+0000',undef,undef,'IRT',undef],
	'Asia/Thimphu'                 => ['+0600','+0000',undef,undef,'BTT',undef],
	'Asia/Tokyo'                   => ['+0900','+0000',undef,undef,'JST',undef],
	'Asia/Ujung_Pandang'           => ['+0800','+0000',undef,undef,'CIT',undef],
	'Asia/Ulaanbaatar'             => ['+0800','+0000',undef,undef,'ULAT',undef],
	'Asia/Urumqi'                  => ['+0800','+0000',undef,undef,'CST',undef],
	'Asia/Vientiane'               => ['+0700','+0000',undef,undef,'ICT',undef],
	'Asia/Vladivostok'             => ['+1000','+0100','-1;0;3;+0200','-1;0;10;+0300','VLAT','VLAST'],
	'Asia/Yakutsk'                 => ['+0900','+0100','-1;0;3;+0200','-1;0;10;+0300','YAKT','YAKST'],
	'Asia/Yekaterinburg'           => ['+0500','+0100','-1;0;3;+0200','-1;0;10;+0300','YEKT','YEKST'],
	'Asia/Yerevan'                 => ['+0400','+0100','-1;0;3;+0200','-1;0;10;+0300','AMT','AMST'],
	'Atlantic/Azores'              => ['-0100','+0100','-1;0;3;+0000','-1;0;10;+0100','AZOT','AZOST'],
	'Atlantic/Bermuda'             => ['-0400','+0100','1;0;4;+0200','-1;0;10;+0200','AST','ADT'],
	'Atlantic/Canary'              => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','WET','WEST'],
	'Atlantic/Cape_Verde'          => ['-0100','+0000',undef,undef,'CVT',undef],
	'Atlantic/Faeroe'              => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','WET','WEST'],
	'Atlantic/Jan_Mayen'           => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Atlantic/Madeira'             => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','WET','WEST'],
	'Atlantic/Reykjavik'           => ['+0000','+0000',undef,undef,'GMT',undef],
	'Atlantic/South_Georgia'       => ['-0200','+0000',undef,undef,'GST',undef],
	'Atlantic/St_Helena'           => ['+0000','+0000',undef,undef,'GMT',undef],
	'Atlantic/Stanley'             => ['-0400','+0100','1;0;9;+0200','3;0;4;+0200','FKT','FKST'],
	'Australia/Adelaide'           => ['+0930','+0100','-1;0;10;+0200','-1;0;3;+0300','CST','CST'],
	'Australia/Brisbane'           => ['+1000','+0000',undef,undef,'EST',undef],
	'Australia/Broken_Hill'        => ['+0930','+0100','-1;0;10;+0200','-1;0;3;+0300','CST','CST'],
	'Australia/Darwin'             => ['+0930','+0000',undef,undef,'CST',undef],
	'Australia/Hobart'             => ['+1000','+0100','1;0;10;+0200','-1;0;3;+0300','EST','EST'],
	'Australia/Lindeman'           => ['+1000','+0000',undef,undef,'EST',undef],
	'Australia/Lord_Howe'          => ['+1030','+0030','-1;0;10;+0200','-1;0;3;+0200','LHST','LHST'],
	'Australia/Melbourne'          => ['+1000','+0100','-1;0;10;+0200','-1;0;3;+0300','EST','EST'],
	'Australia/Perth'              => ['+0800','+0000',undef,undef,'WST',undef],
	'Australia/Sydney'             => ['+1000','+0100','-1;0;10;+0200','-1;0;3;+0300','EST','EST'],
	'Europe/Amsterdam'             => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Andorra'               => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Athens'                => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Belfast'               => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','GMT','BST'],
	'Europe/Belgrade'              => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Berlin'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Bratislava'            => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Brussels'              => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Bucharest'             => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Budapest'              => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Chisinau'              => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Copenhagen'            => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Dublin'                => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','GMT','IST'],
	'Europe/Gibraltar'             => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Helsinki'              => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Istanbul'              => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Kaliningrad'           => ['+0200','+0100','-1;0;3;+0200','-1;0;10;+0300','EET','EEST'],
	'Europe/Kiev'                  => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Lisbon'                => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','WET','WEST'],
	'Europe/Ljubljana'             => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/London'                => ['+0000','+0100','-1;0;3;+0100','-1;0;10;+0200','GMT','BST'],
	'Europe/Luxembourg'            => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Madrid'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Malta'                 => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Minsk'                 => ['+0200','+0100','-1;0;3;+0200','-1;0;10;+0300','EET','EEST'],
	'Europe/Monaco'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Moscow'                => ['+0300','+0100','-1;0;3;+0200','-1;0;10;+0300','MSK','MSD'],
	'Europe/Nicosia'               => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Oslo'                  => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Paris'                 => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Prague'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Riga'                  => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Rome'                  => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Samara'                => ['+0400','+0100','-1;0;3;+0200','-1;0;10;+0300','SAMT','SAMST'],
	'Europe/San_Marino'            => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Sarajevo'              => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Simferopol'            => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Skopje'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Sofia'                 => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Stockholm'             => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Tallinn'               => ['+0200','+0000',undef,undef,'EET',undef],
	'Europe/Tirane'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Uzhgorod'              => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Vaduz'                 => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Vatican'               => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Vienna'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Vilnius'               => ['+0200','+0000',undef,undef,'EET',undef],
	'Europe/Warsaw'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Zagreb'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Europe/Zaporozhye'            => ['+0200','+0100','-1;0;3;+0300','-1;0;10;+0400','EET','EEST'],
	'Europe/Zurich'                => ['+0100','+0100','-1;0;3;+0200','-1;0;10;+0300','CET','CEST'],
	'Indian/Antananarivo'          => ['+0300','+0000',undef,undef,'EAT',undef],
	'Indian/Chagos'                => ['+0600','+0000',undef,undef,'IOT',undef],
	'Indian/Christmas'             => ['+0700','+0000',undef,undef,'CXT',undef],
	'Indian/Cocos'                 => ['+0630','+0000',undef,undef,'CCT',undef],
	'Indian/Comoro'                => ['+0300','+0000',undef,undef,'EAT',undef],
	'Indian/Kerguelen'             => ['+0500','+0000',undef,undef,'TFT',undef],
	'Indian/Mahe'                  => ['+0400','+0000',undef,undef,'SCT',undef],
	'Indian/Maldives'              => ['+0500','+0000',undef,undef,'MVT',undef],
	'Indian/Mauritius'             => ['+0400','+0000',undef,undef,'MUT',undef],
	'Indian/Mayotte'               => ['+0300','+0000',undef,undef,'EAT',undef],
	'Indian/Reunion'               => ['+0400','+0000',undef,undef,'RET',undef],
	'Pacific/Apia'                 => ['-1100','+0000',undef,undef,'WST',undef],
	'Pacific/Auckland'             => ['+1200','+0100','1;0;10;+0200','3;0;3;+0300','NZST','NZDT'],
	'Pacific/Chatham'              => ['+1245','+0100','1;0;10;+0245','3;0;3;+0345','CHAST','CHADT'],
	'Pacific/Easter'               => ['-0600','+0100','2;6;10;+2200','2;6;3;+2200','EAST','EASST'],
	'Pacific/Efate'                => ['+1100','+0000',undef,undef,'VUT',undef],
	'Pacific/Enderbury'            => ['+1300','+0000',undef,undef,'PHOT',undef],
	'Pacific/Fakaofo'              => ['-1000','+0000',undef,undef,'TKT',undef],
	'Pacific/Fiji'                 => ['+1200','+0000',undef,undef,'FJT',undef],
	'Pacific/Funafuti'             => ['+1200','+0000',undef,undef,'TVT',undef],
	'Pacific/Galapagos'            => ['-0600','+0000',undef,undef,'GALT',undef],
	'Pacific/Gambier'              => ['-0900','+0000',undef,undef,'GAMT',undef],
	'Pacific/Guadalcanal'          => ['+1100','+0000',undef,undef,'SBT',undef],
	'Pacific/Guam'                 => ['+1000','+0000',undef,undef,'ChST',undef],
	'Pacific/Honolulu'             => ['-1000','+0000',undef,undef,'HST',undef],
	'Pacific/Johnston'             => ['-1000','+0000',undef,undef,'HST',undef],
	'Pacific/Kiritimati'           => ['+1400','+0000',undef,undef,'LINT',undef],
	'Pacific/Kosrae'               => ['+1100','+0000',undef,undef,'KOST',undef],
	'Pacific/Kwajalein'            => ['+1200','+0000',undef,undef,'MHT',undef],
	'Pacific/Majuro'               => ['+1200','+0000',undef,undef,'MHT',undef],
	'Pacific/Marquesas'            => ['-0930','+0000',undef,undef,'MART',undef],
	'Pacific/Midway'               => ['-1100','+0000',undef,undef,'SST',undef],
	'Pacific/Nauru'                => ['+1200','+0000',undef,undef,'NRT',undef],
	'Pacific/Niue'                 => ['-1100','+0000',undef,undef,'NUT',undef],
	'Pacific/Norfolk'              => ['+1130','+0000',undef,undef,'NFT',undef],
	'Pacific/Noumea'               => ['+1100','+0000',undef,undef,'NCT',undef],
	'Pacific/Pago_Pago'            => ['-1100','+0000',undef,undef,'SST',undef],
	'Pacific/Palau'                => ['+0900','+0000',undef,undef,'PWT',undef],
	'Pacific/Pitcairn'             => ['-0800','+0000',undef,undef,'PST',undef],
	'Pacific/Ponape'               => ['+1100','+0000',undef,undef,'PONT',undef],
	'Pacific/Port_Moresby'         => ['+1000','+0000',undef,undef,'PGT',undef],
	'Pacific/Rarotonga'            => ['-1000','+0000',undef,undef,'CKT',undef],
	'Pacific/Saipan'               => ['+1000','+0000',undef,undef,'ChST',undef],
	'Pacific/Tahiti'               => ['-1000','+0000',undef,undef,'TAHT',undef],
	'Pacific/Tarawa'               => ['+1200','+0000',undef,undef,'GILT',undef],
	'Pacific/Tongatapu'            => ['+1300','+0000',undef,undef,'TOT',undef],
	'Pacific/Truk'                 => ['+1000','+0000',undef,undef,'TRUT',undef],
	'Pacific/Wake'                 => ['+1200','+0000',undef,undef,'WAKT',undef],
	'Pacific/Wallis'               => ['+1200','+0000',undef,undef,'WFT',undef],
	'Pacific/Yap'                  => ['+1000','+0000',undef,undef,'YAPT',undef],
);
1;
__END__
